part of play;

class PlayVideo extends StatefulWidget {
  PlayVideo({
    super.key,
  });

  @override
  State<PlayVideo> createState() => PlayVideoState();
}

class PlayVideoState extends State<PlayVideo> {
  late dart_vlc.Player desktopPlayer;
  late video_player.VideoPlayerController mobilePlayer;
  late bool isInit = false;

  @override
  void initState() {
    super.initState();
    initialize();
  }

  @override
  void dispose() {
    mobilePlayer.dispose();
    desktopPlayer.dispose();
    super.dispose();
  }

  Future<void> initialize() async {
    if (Platform.isWindows || Platform.isLinux) {
      setState(() {
        desktopPlayer = dart_vlc.Player(
            id: 0,
            videoDimensions: const dart_vlc.VideoDimensions(640, 460),
            registerTexture: !Platform.isWindows);
      });
      isInit = true;
    } else if (Platform.isAndroid || Platform.isIOS) {
      setState(() {
        mobilePlayer = video_player.VideoPlayerController.asset(
            "assets/video/azkadev.mp4");
      });
      await mobilePlayer.initialize();
      setState(() {});
      isInit = true;
      setState(() {});
    }
  }

  void playFromFile(File file) {
    if (Platform.isWindows || Platform.isLinux) {
      desktopPlayer.open(
        dart_vlc.Playlist(
          medias: [
            dart_vlc.Media.file(file),
          ],
          playlistMode: dart_vlc.PlaylistMode.single,
        ),
      );
    }
  }

  FutureOr<bool> openNetwork(String url) async {
    if (Platform.isWindows || Platform.isLinux) {
      desktopPlayer.open(
        dart_vlc.Playlist(
          medias: [
            dart_vlc.Media.network(url),
          ],
          playlistMode: dart_vlc.PlaylistMode.single,
        ),
      );
    } else if (Platform.isAndroid || Platform.isIOS) {
      mobilePlayer = video_player.VideoPlayerController.network(url);
      await mobilePlayer.initialize();
    }
    return false;
  }

  FutureOr<bool> openFile(File file) async {
    if (Platform.isWindows || Platform.isLinux) {
      desktopPlayer.open(
        dart_vlc.Playlist(
          medias: [
            dart_vlc.Media.file(file),
          ],
          playlistMode: dart_vlc.PlaylistMode.single,
        ),
      );
    } else if (Platform.isAndroid || Platform.isIOS) {
      mobilePlayer = video_player.VideoPlayerController.file(file);
      await mobilePlayer.initialize();
    }
    return false;
  }

  Duration getDurationMax() {
    if (Platform.isWindows || Platform.isLinux) {
      return desktopPlayer.position.duration ?? Duration();
    }
    return Duration();
  }

  Duration getDurationPosition() {
    if (Platform.isWindows || Platform.isLinux) {
      return desktopPlayer.position.position ?? Duration();
    }
    return Duration();
  }

  streamDurationPosition() {
    if (Platform.isWindows || Platform.isLinux) {
      return desktopPlayer.positionStream;
    } else if (Platform.isAndroid || Platform.isIOS) {}
    return Stream.value("ok");
  }

  seek(Duration duration) {
    if (Platform.isWindows || Platform.isLinux) {
      desktopPlayer.seek(duration);
    } else if (Platform.isAndroid || Platform.isIOS) {
      mobilePlayer.seekTo(duration);
    }
  }

  playOrPause() {
    if (Platform.isWindows || Platform.isLinux) {
      desktopPlayer.playOrPause();
    }
    if (Platform.isAndroid || Platform.isIOS) {
      // mobilePlayer. ? mobilePlayer.pause() : mobilePlayer.play();
    }
  }

  @override
  Widget build(BuildContext context) {
    if (Platform.isWindows || Platform.isLinux) {
      return Visibility(
        visible: isInit,
        replacement: frame(Container()),
        child: frame(
          dart_vlc.Video(
            player: desktopPlayer,
            width: desktopPlayer.videoDimensions.width.toDouble(),
            height: desktopPlayer.videoDimensions.height.toDouble(),
            volumeThumbColor: Colors.blue,
            volumeActiveColor: Colors.blue,
            showControls: false,
            showFullscreenButton: true,
            showTimeLeft: true,
            fillColor: Colors.black,
          ),
        ),
      );
    } else {
      return Visibility(
        visible: isInit,
        replacement: frame(Container()),
        child: frame(
          video_player.VideoPlayer(mobilePlayer),
        ),
      );
    }
  }

  Widget frame(Widget child) {
    if (Platform.isWindows || Platform.isLinux) {
      return Container(
        height: 512,
        width: 512,
        decoration: BoxDecoration(
          color: Colors.black,
          borderRadius: BorderRadius.all(
            Radius.circular(20),
          ),
        ),
        child: Stack(
          fit: StackFit.passthrough,
          children: [
            child,
            Positioned(
              bottom: 5,
              left: 0,
              right: 0,
              child: Padding(
                padding: const EdgeInsets.symmetric(horizontal: 20),
                child: Row(
                  children: [
                    InkWell(
                      onTap: () {
                        playOrPause();
                        setState(() {});
                      },
                      child: const Icon(
                        Icons.pause,
                        color: Colors.white,
                      ),
                    ),
                    InkWell(
                      onTap: () {},
                      child: const Icon(
                        Icons.skip_next,
                        color: Colors.white,
                      ),
                    ),
                    Expanded(
                      child: StreamBuilder(
                        stream: streamDurationPosition(),
                        builder:
                            (BuildContext context, AsyncSnapshot snapshot) {
                          return Slider(
                            min: 0,
                            max: getDurationMax().inMilliseconds.toDouble(),
                            value:
                                getDurationPosition().inMilliseconds.toDouble(),
                            onChanged: (double value) {
                              setState(() {
                                seek(Duration(milliseconds: value.toInt()));
                              });
                            },
                          );
                        },
                      ),
                    ),
                    InkWell(
                      onTap: () {},
                      child: const Icon(
                        Icons.fullscreen,
                        color: Colors.white,
                      ),
                    ),
                  ],
                ),
              ),
            ),
          ],
        ),
      );
    }
    return child;
  }
}
